@@ -502,6 +502,14 @@ class Manager(manager.Manager):
 
         driver = self._select_identity_driver(domain_id)
 
+        if not driver.is_sql:
+            # The LDAP driver does not support deleting users or groups.
+            # Moreover, we shouldn't destroy users and groups in an unknown
+            # driver. The only time when we should delete users and groups is
+            # when the backend is SQL because the foreign key in the SQL table
+            # forces us to.
+            return
+
         user_refs = self.list_users(domain_scope=domain_id)
         group_refs = self.list_groups(domain_scope=domain_id)
 
@@ -518,10 +526,7 @@ class Manager(manager.Manager):
         # And finally, delete the users themselves
         for user in user_refs:
             try:
-                if not driver.is_sql:
-                    PROVIDERS.shadow_users_api.delete_user(user['id'])
-                else:
-                    self.delete_user(user['id'])
+                self.delete_user(user['id'])
             except exception.UserNotFound:
                 LOG.debug(('User %(userid)s not found when deleting domain '
                            'contents for %(domainid)s, continuing with '
@@ -626,7 +631,10 @@ class Manager(manager.Manager):
         LOG.debug('Local ID: %s', ref['id'])
         ref = ref.copy()
 
-        self._insert_domain_id_if_needed(ref, driver, domain_id, conf)
+        if not driver.is_domain_aware():
+            if not domain_id:
+                domain_id = CONF.identity.default_domain_id
+            ref['domain_id'] = domain_id
 
         if self._is_mapping_needed(driver):
             local_entity = {'domain_id': ref['domain_id'],
@@ -650,8 +658,15 @@ class Manager(manager.Manager):
         if not ref_list:
             return []
 
-        for r in ref_list:
-            self._insert_domain_id_if_needed(r, driver, domain_id, conf)
+        # If the domain_id is None that means we are running in a single
+        # backend mode, so to remain backwards compatible we will use the
+        # default domain ID.
+        if not domain_id:
+            domain_id = CONF.identity.default_domain_id
+
+        if not driver.is_domain_aware():
+            for ref in ref_list:
+                ref['domain_id'] = domain_id
 
         if not self._is_mapping_needed(driver):
             return ref_list
@@ -661,17 +676,10 @@ class Manager(manager.Manager):
         for r in ref_list:
             refs_map[(r['id'], entity_type, r['domain_id'])] = r
 
-        # NOTE(breton): there are cases when the driver is not domain aware and
-        # no domain_id was explicitely provided for list operation. domain_id
-        # gets inserted into refs, but not passed into this method. Lets use
-        # domain_id from one of the refs.
-        if not domain_id:
-            domain_id = ref_list[0]['domain_id']
-
         # fetch all mappings for the domain, lookup the user at the map built
         # at previous step and replace his id.
         domain_mappings = PROVIDERS.id_mapping_api.get_domain_mapping_list(
-            domain_id)
+            domain_id, entity_type=entity_type)
         for _mapping in domain_mappings:
             idx = (_mapping.local_id, _mapping.entity_type, _mapping.domain_id)
             try:
@@ -692,19 +700,6 @@ class Manager(manager.Manager):
             self._insert_new_public_id(local_entity, ref, driver)
         return ref_list
 
-    def _insert_domain_id_if_needed(self, ref, driver, domain_id, conf):
-        """Insert the domain ID into the ref, if required.
-
-        If the driver can't handle domains, then we need to insert the
-        domain_id into the entity being returned.  If the domain_id is
-        None that means we are running in a single backend mode, so to
-        remain backwardly compatible, we put in the default domain ID.
-        """
-        if not driver.is_domain_aware():
-            if domain_id is None:
-                domain_id = conf.default_domain_id
-            ref['domain_id'] = domain_id
-
     def _is_mapping_needed(self, driver):
         """Return whether mapping is needed.
 
@@ -1034,22 +1029,19 @@ class Manager(manager.Manager):
         return hints
 
     def _handle_shadow_and_local_users(self, driver, hints):
-        federated_attributes = ['idp_id', 'protocol_id', 'unique_id']
+        federated_attributes = {'idp_id', 'protocol_id', 'unique_id'}
+        fed_res = []
         for filter_ in hints.filters:
             if filter_['name'] in federated_attributes:
                 return PROVIDERS.shadow_users_api.get_federated_users(hints)
-        fed_hints = copy.deepcopy(hints)
-        res = driver.list_users(hints)
-
-        # Note: If the filters contain 'name', we should get the user from both
-        # local user and shadow user backend.
-        for filter_ in fed_hints.filters:
+            # Note: If the filters contain 'name', we should get the user from
+            # both local user and shadow user backend.
             if filter_['name'] == 'name':
+                fed_hints = copy.deepcopy(hints)
                 fed_res = PROVIDERS.shadow_users_api.get_federated_users(
                     fed_hints)
-                res += fed_res
                 break
-        return res
+        return driver.list_users(hints) + fed_res
 
     @domains_configured
     @exception_translated('user')
@@ -1123,7 +1115,14 @@ class Manager(manager.Manager):
         enabled_change = ((user.get('enabled') is False) and
                           user['enabled'] != old_user_ref.get('enabled'))
         if enabled_change or user.get('password') is not None:
-            self.emit_invalidate_user_token_persistence(user_id)
+            self._persist_revocation_event_for_user(user_id)
+            reason = (
+                'Invalidating the token cache because user %(user_id)s was '
+                'enabled or disabled. Authorization will be calculated and '
+                'enforced accordingly the next time they authenticate or '
+                'validate a token.' % {'user_id': user_id}
+            )
+            notifications.invalidate_token_cache_notification(reason)
 
         return self._set_domain_id_and_mapping(
             ref, domain_id, driver, mapping.EntityType.USER)
@@ -1135,11 +1134,22 @@ class Manager(manager.Manager):
             self._get_domain_driver_and_entity_id(user_id))
         # Get user details to invalidate the cache.
         user_old = self.get_user(user_id)
+
+        hints = driver_hints.Hints()
+        hints.add_filter('user_id', user_id)
+        fed_users = PROVIDERS.shadow_users_api.list_federated_users_info(hints)
+
         driver.delete_user(entity_id)
         PROVIDERS.assignment_api.delete_user_assignments(user_id)
         self.get_user.invalidate(self, user_id)
         self.get_user_by_name.invalidate(self, user_old['name'],
                                          user_old['domain_id'])
+        for fed_user in fed_users:
+            self.shadow_federated_user.invalidate(
+                self, fed_user['idp_id'], fed_user['protocol_id'],
+                fed_user['unique_id'], fed_user['display_name'],
+                user_old.get('extra', {}).get('email'))
+
         PROVIDERS.credential_api.delete_credentials_for_user(user_id)
         PROVIDERS.id_mapping_api.delete_id_mapping(user_id)
         notifications.Audit.deleted(self._USER, user_id, initiator)
@@ -1226,8 +1236,8 @@ class Manager(manager.Manager):
         # assignment for the group then we do not need to revoke all the users
         # tokens and can just delete the group.
         if roles:
-            for uid in user_ids:
-                self.emit_invalidate_user_token_persistence(uid)
+            for user_id in user_ids:
+                self._persist_revocation_event_for_user(user_id)
 
         # Invalidate user role assignments cache region, as it may be caching
         # role assignments expanded from the specified group to its users
@@ -1276,7 +1286,7 @@ class Manager(manager.Manager):
             user_entity_id, user_driver, group_entity_id, group_driver)
 
         group_driver.remove_user_from_group(user_entity_id, group_entity_id)
-        self.emit_invalidate_user_token_persistence(user_id)
+        self._persist_revocation_event_for_user(user_id)
 
         # Invalidate user role assignments cache region, as it may be caching
         # role assignments expanded from this group to this user
@@ -1284,31 +1294,17 @@ class Manager(manager.Manager):
         notifications.Audit.removed_from(self._GROUP, group_id, self._USER,
                                          user_id, initiator)
 
-    def emit_invalidate_user_token_persistence(self, user_id):
-        """Emit a notification to the callback system to revoke user tokens.
+    def _persist_revocation_event_for_user(self, user_id):
+        """Emit a notification to invoke a revocation event callback.
 
-        This method and associated callback listener removes the need for
-        making a direct call to another manager to delete and revoke tokens.
+        Fire off an internal notification that will be consumed by the
+        revocation API to store a revocation record for a specific user.
 
         :param user_id: user identifier
         :type user_id: string
         """
         notifications.Audit.internal(
-            notifications.INVALIDATE_USER_TOKEN_PERSISTENCE, user_id
-        )
-
-    def emit_invalidate_grant_token_persistence(self, user_project):
-        """Emit a notification to the callback system to revoke grant tokens.
-
-        This method and associated callback listener removes the need for
-        making a direct call to another manager to delete and revoke tokens.
-
-        :param user_project: {'user_id': user_id, 'project_id': project_id}
-        :type user_project: dict
-        """
-        notifications.Audit.internal(
-            notifications.INVALIDATE_USER_PROJECT_TOKEN_PERSISTENCE,
-            user_project
+            notifications.PERSIST_REVOCATION_EVENT_FOR_USER, user_id
         )
 
     @domains_configured
@@ -1403,7 +1399,7 @@ class Manager(manager.Manager):
             raise
 
         notifications.Audit.updated(self._USER, user_id, initiator)
-        self.emit_invalidate_user_token_persistence(user_id)
+        self._persist_revocation_event_for_user(user_id)
 
     @MEMOIZE
     def _shadow_nonlocal_user(self, user):
@@ -1414,13 +1410,14 @@ class Manager(manager.Manager):
 
     @MEMOIZE
     def shadow_federated_user(self, idp_id, protocol_id, unique_id,
-                              display_name):
+                              display_name, email=None):
         """Map a federated user to a user.
 
         :param idp_id: identity provider id
         :param protocol_id: protocol id
         :param unique_id: unique id for the user within the IdP
         :param display_name: user's display name
+        :param email: user's email
 
         :returns: dictionary of the mapped User entity
         """
@@ -1430,6 +1427,10 @@ class Manager(manager.Manager):
                 idp_id, protocol_id, unique_id, display_name)
             user_dict = PROVIDERS.shadow_users_api.get_federated_user(
                 idp_id, protocol_id, unique_id)
+            if email:
+                user_ref = {"email": email}
+                self.update_user(user_dict['id'], user_ref)
+                user_dict.update({"email": email})
         except exception.UserNotFound:
             idp = PROVIDERS.federation_api.get_idp(idp_id)
             federated_dict = {
@@ -1440,7 +1441,7 @@ class Manager(manager.Manager):
             }
             user_dict = (
                 PROVIDERS.shadow_users_api.create_federated_user(
-                    idp['domain_id'], federated_dict
+                    idp['domain_id'], federated_dict, email=email
                 )
             )
         PROVIDERS.shadow_users_api.set_last_active_at(user_dict['id'])
