@@ -294,15 +294,10 @@ class TokenModel(object):
             set([r['role_id'] for r in effective_trust_roles])
         )
 
-        trustor_assignments = (
-            PROVIDERS.assignment_api.list_role_assignments(
-                user_id=original_trustor_id,
-                project_id=self.trust.get('project_id'),
-                effective=True, strip_domain_roles=False
-            )
-        )
         current_effective_trustor_roles = (
-            set([x['role_id'] for x in trustor_assignments])
+            PROVIDERS.assignment_api.get_roles_for_trustor_and_project(
+                original_trustor_id, self.trust.get('project_id')
+            )
         )
 
         for trust_role_id in effective_trust_role_ids:
@@ -505,13 +500,10 @@ class TokenModel(object):
             effective_trust_role_ids = (
                 set([r['role_id'] for r in effective_trust_roles])
             )
-            assignments = PROVIDERS.assignment_api.list_role_assignments(
-                user_id=self.trustor['id'], system=self.system,
-                project_id=self.project_id, effective=True,
-                strip_domain_roles=False
-            )
             current_effective_trustor_roles = (
-                set([x['role_id'] for x in assignments])
+                PROVIDERS.assignment_api.get_roles_for_trustor_and_project(
+                    self.trustor['id'], self.trust.get('project_id')
+                )
             )
             # Go through each of the effective trust roles, making sure the
             # trustor still has them, if any have been removed, then we
